<?php
// $Id: drupalerp.profile,v 1.1.2.1 2009/12/01 10:11:30 simon Exp $


/**
 * Return a description of the profile for the initial installation screen.
 *
 * @return
 *   An array with keys 'name' and 'description' describing this profile.
 */
function erp_profile_details() {
  return array(
    'name' => 'ERP',
    'description' => 'This installation profile will setup a basic ERP system.'
  );
}

/**
 * Return an array of the modules to be enabled when this profile is installed.
 *
 * @return
 *  An array of modules to be enabled.
 */
function erp_profile_modules() {
  return array(
    // Core drupal modules
    'comment', 'help', 'menu', 'taxonomy', 'book', 'search', 'upload', 'dblog',

    // Other contrib modules we use
    'context', 'context_ui', 'date_api', 'date_popup', 'date_timezone', 'gmap', 'quicktabs', 'print', 'print_mail', 'print_pdf', 'views', 'views_ui',

    // Now the erp modules - core
    'erp', 'erp_cart', 'erp_store',

    // Accounting/sales
    'erp_accounting', 'erp_tax', 'erp_stock', 'erp_quote', 'erp_cash_sale',
    'erp_invoice', 'erp_purchase_order', 'erp_item',
    'erp_goods_receive', 'erp_payment', 'erp_supplier',
    'erp_recurring', 'erp_pricing',

    // Customer
    'erp_contact', 'erp_customer',

    // Job modules
    'erp_job',

    // Auxillary utilities
    'erp_franchise',
  );
}

function erp_profile_task_list() {
	 return array(
	   'config' => 'Configure ERP',
	   'sample' => 'Install sample data?',
	 );
}

function erp_profile_tasks(&$task, $url) {
	include_once './includes/form.inc';

  switch ($task) {
  	case 'profile':
      _erp_profile_std_setup();

      $task = 'config';
      drupal_set_title(t('Configure erp'));
      return drupal_get_form('_erp_profile_config_erp_form', $url);
      break;

  	case 'config':
      _erp_profile_config_form_submit($form, &$form_state);

      $task = 'sample';
      drupal_set_title(t('Install sample data?'));
      return drupal_get_form('_erp_profile_sample_erp_form', $url);
  		break;

  	case 'sample':
  		_erp_profile_sample_form_submit($form, &$form_state);

      // Move to the completion task.
      $task = 'profile-finished';
  		break;
  }
}

/**
 *
 */
function _erp_profile_config_erp_form($form_state, $url) {
  $form = array(
    '#action' => $url,
    '#redirect' => FALSE,
  );

  $form['erp_default_store_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Store information'),
    '#tree' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_name'] = array(
    '#type' => 'textfield',
    '#title' => t('Store name'),
    '#description' => NULL,
    '#size' => 32,
    '#maxlength' => 128,
    '#default_value' => variable_get('site_name', ''),
    '#required' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_street_address'] = array(
    '#type' => 'textfield',
    '#title' => t('Store address'),
    '#description' => NULL,
    '#size' => 64,
    '#maxlength' => 128,
    '#required' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_suburb'] = array(
    '#type' => 'textfield',
    '#title' => t('Store suburb'),
    '#description' => NULL,
    '#size' => 32,
    '#maxlength' => 128,
    '#required' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_state'] = array(
    '#type' => 'textfield',
    '#title' => t('Store state'),
    '#description' => NULL,
    '#size' => 10,
    '#maxlength' => 10,
    '#required' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_postcode'] = array(
    '#type' => 'textfield',
    '#title' => t('Store postcode'),
    '#description' => NULL,
    '#size' => 12,
    '#maxlength' => 12,
    '#required' => TRUE,
  );

  $form['erp_default_store_info']['erp_store_phone'] = array(
    '#type' => 'textfield',
    '#title' => t('Store phone'),
    '#description' => NULL,
    '#size' => 20,
    '#maxlength' => 20,
    '#required' => TRUE,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save and continue'),
  );

  return $form;
}

/**
 * Setup specific standard/common parts of erp
 *
 */
function _erp_profile_config_form_submit() {
  module_load_include('inc', 'node', 'node.pages');

  // Check_plain on vars
  $values = $_POST['erp_default_store_info'];
  foreach ($values as $field => $value) {
  	$values[$field] = check_plain($value);
  }
  
  drupal_get_schema(NULL, TRUE);

  $default_user = db_result(db_query("SELECT name FROM {users} WHERE uid = 1"));

  // Create default customer book, default supplier book, default store book
  $node = array();
  $node['type'] = 'book'; // node type

  $form_state = array();
  $form_state['values']['title'] = t('Customer Documentation'); // node title
  $form_state['values']['body'] = t('This book is used to contain all customer documentation'); // node body
  $form_state['values']['taxonomy'] = array('tags' => array('1' => ('Customer Documentation')));
  $form_state['values']['name'] = $default_user;
  $form_state['values']['book-bid'] = 1; // Create new book

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('book_node_form', $form_state, (object)$node);

  variable_set('erp_customer_doc_book', $form_state['nid']);
  variable_get('erp_customer_doc_parent', $form_state['nid']);

  $node = array();
  $node['type'] = 'book'; // node type

  $form_state = array();
  $form_state['values']['title'] = t('Supplier Documentation'); // node title
  $form_state['values']['body'] = t('This book is used to contain all supplier documentation'); // node body
  $form_state['values']['taxonomy'] = array('tags' => array('1' => ('Supplier Documentation')));
  $form_state['values']['name'] = $default_user;
  $form_state['values']['book-bid'] = 1; // Create new book

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('book_node_form', $form_state, (object)$node);

  variable_set('erp_supplier_doc_book', $form_state['nid']);
  variable_get('erp_supplier_doc_parent', $form_state['nid']);

  $node = array();
  $node['type'] = 'book'; // node type
  $form_state = array();

  $form_state['values']['title'] = t('Store Documentation'); // node title
  $form_state['values']['body'] = t('This book is used to contain all store documentation'); // node body
  $form_state['values']['taxonomy'] = array('tags' => array('1' => ('Store Documentation')));
  $form_state['values']['name'] = $default_user;
  $form_state['values']['book-bid'] = 1; // Create new book

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('book_node_form', $form_state, (object)$node);

  variable_set('erp_store_doc_book', $form_state['nid']);
  variable_get('erp_store_doc_parent', $form_state['nid']);

  // Create a store and make it the default
  $node = array();
  $node['type'] = 'erp_store'; // node type

  $form_state = array();
  $form_state['values']['title'] = $values['erp_store_name'];
  $form_state['values']['company'] = $values['erp_store_name'];
  $form_state['values']['address'] = $values['erp_store_street_address'];
  $form_state['values']['suburb'] = $values['erp_store_suburb'];
  $form_state['values']['state'] = $values['erp_store_state'];
  $form_state['values']['postcode'] = $values['erp_store_postcode'];
  $form_state['values']['phone'] = $values['erp_store_phone'];
  $form_state['values']['name'] = $default_user;

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('erp_store_node_form', $form_state, (object)$node);

  variable_set('erp_default_store', $form_state['nid']);

  // Create default customer "test"
  $node = array();
  $node['type'] = 'erp_customer'; // node type

  $form_state = array();
  $form_state['values']['title'] = 'Test customer'; // node title
  $form_state['values']['body'] = 'Test customer'; // node body
  $form_state['values']['address'] = 'Test customer address';
  $form_state['values']['phone'] = '+40-6-5000'; // Transylvania 6-5000!
  $form_state['values']['name'] = $default_user;

  $form_state['values']['franchisee'] = $default_user;

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('erp_customer_node_form', $form_state, (object)$node);

  // Create default supplier "internal"
  $node = array();
  $node['type'] = 'erp_supplier'; // node type

  $internal_supplier = $values['erp_store_name'] .' '. t('Supplier');

  $form_state = array();
  $form_state['values']['title'] = $internal_supplier;
  $form_state['values']['body'] = 'Internal use supplier'; // node body
  $form_state['values']['company'] = $values['erp_store_name'];
  $form_state['values']['address'] = $values['erp_store_street_address'];
  $form_state['values']['suburb'] = $values['erp_store_suburb'];
  $form_state['values']['state'] = $values['erp_store_state'];
  $form_state['values']['postcode'] = $values['erp_store_postcode'];
  $form_state['values']['phone'] = $values['erp_store_phone'];
  $form_state['values']['name'] = $default_user;

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('erp_supplier_node_form', $form_state, (object)$node);

  // Create default item "LABOUR"
  $node = array();
  $node['type'] = 'erp_item'; // node type

  $form_state = array();
  $form_state['values']['title'] = t('LABOUR'); // node title
  $form_state['values']['code'] = t('LABOUR');
  $form_state['values']['supp_code'] = t('LABOUR');
  $form_state['values']['supplier'] = $internal_supplier;
  $form_state['values']['item_type'] = 'service';
  $form_state['values']['item_locked'] = 1;
  $form_state['values']['active'] = 1;
  $form_state['values']['full_desc'] = t('Use this item to charge our labour to clients');
  $form_state['values']['rrp_price'] = 110;
  $form_state['values']['sell_price'] = 110;
  $form_state['values']['buy_price'] = 0;
  $form_state['values']['name'] = $default_user;

  $form_state['values']['op'] = t('Save'); // required
  drupal_execute('erp_item_node_form', $form_state, (object)$node);

  // Hide the user login block
  db_query("UPDATE {blocks} SET status = 0 WHERE module = 'user' AND delta = 0");

  // Setup erp customer search
  db_query("UPDATE {blocks} SET status = 1, region = 'left', weight = 1, theme = 'bluemarine' WHERE module = 'erp'");
  db_query("UPDATE {blocks} SET status = 1, region = 'left', weight = 2, theme = 'bluemarine' WHERE module = 'erp_customer'");
  db_query("UPDATE {blocks} SET status = 1, region = 'left', weight = 3, theme = 'bluemarine' WHERE module = 'erp_cart'");

  // Put admin block on right
  db_query("UPDATE {blocks} SET region = 'right' WHERE module = 'user' AND delta = 1");

  db_query("INSERT INTO `quicktabs` (`qtid`, `ajax`, `title`, `tabs`, `style`) VALUES (1, 0, 'customer tabs', '%s', 'Basic')",
    'a:7:{i:0;a:5:{s:3:"bid";s:40:"views_delta_erp_invoice_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:8:"invoices";s:6:"weight";s:3:"-10";s:4:"type";s:5:"block";}i:1;a:5:{s:3:"bid";s:44:"views_delta_4800c5eaa1772297a56142ea0d52c826";s:10:"hide_title";i:1;s:5:"title";s:15:"purchase orders";s:6:"weight";s:2:"-9";s:4:"type";s:5:"block";}i:2;a:5:{s:3:"bid";s:38:"views_delta_erp_quote_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:6:"quotes";s:6:"weight";s:2:"-8";s:4:"type";s:5:"block";}i:3;a:5:{s:3:"bid";s:36:"views_delta_erp_job_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:4:"jobs";s:6:"weight";s:2:"-7";s:4:"type";s:5:"block";}i:4;a:5:{s:3:"bid";s:42:"views_delta_erp_cash_sale_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:10:"cash sales";s:6:"weight";s:2:"-6";s:4:"type";s:5:"block";}i:5;a:5:{s:3:"bid";s:40:"views_delta_erp_payment_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:8:"payments";s:6:"weight";s:2:"-5";s:4:"type";s:5:"block";}i:6;a:5:{s:3:"bid";s:37:"views_delta_erp_book_customer-block_1";s:10:"hide_title";i:1;s:5:"title";s:10:"book pages";s:6:"weight";s:2:"-4";s:4:"type";s:5:"block";}}');

  db_query("INSERT INTO `context` (`cid`, `namespace`, `attribute`, `value`, `data`) VALUES (1, 'erp', 'tabs', 'customer_details', '%s')",
    'a:4:{s:6:"system";i:0;s:11:"description";s:0:"";s:4:"node";a:1:{i:0;s:12:"erp_customer";}s:5:"block";a:1:{s:11:"quicktabs_1";O:8:"stdClass":8:{s:6:"module";s:9:"quicktabs";s:5:"delta";s:1:"1";s:6:"weight";i:20;s:6:"region";s:7:"content";s:6:"status";s:1:"0";s:5:"label";s:13:"customer tabs";s:4:"type";s:10:"context_ui";s:3:"bid";s:11:"quicktabs_1";}}}');
  
  drupal_flush_all_caches();
  drupal_set_message(t('Caches cleared.'));
}

/**
 *
 */
function _erp_profile_sample_erp_form($form_state, $url) {
  $form = array(
    '#action' => $url,
    '#redirect' => FALSE,
  );

  $form['erp_sample_store_info'] = array(
    '#type' => 'fieldset',
    '#title' => t('Sample data setup'),
  );

  $form['erp_default_store_info']['install_sample_data'] = array(
    '#type' => 'select',
    '#title' => t('Install sample data'),
    '#description' => NULL,
    '#options' => array(1 => t('Yes'), 0 => t('No')),
    '#default_value' => 1,
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Save and continue'),
  );

  return $form;
}

/**
 * Actually load the sample data
 *
 */
function _erp_profile_sample_form_submit() {
  // TODO
  // Create some invoices, purchase orders, goods receive etc.
}

/**
 * Perform any final installation tasks for this profile.
 *
 * @return
 *   An optional HTML string to display to the user on the final installation
 *   screen.
 */
function _erp_profile_std_setup() {
  // Default page to not be promoted and have comments disabled.
  variable_set('node_options_page', array('status'));
  variable_set('comment_page', COMMENT_NODE_DISABLED);

  // Don't display date and author information for page nodes by default.
  $theme_settings = variable_get('theme_settings', array());
  $theme_settings['toggle_node_info_page'] = FALSE;

  variable_set('theme_settings', $theme_settings);

  // Bluemarine seems nicest for erp
  db_query("UPDATE {system} SET status = 1 WHERE name = 'bluemarine'");
  variable_set('theme_default', 'bluemarine');

  // Take over garland blocks
  db_query("UPDATE {blocks} SET theme = 'bluemarine' WHERE theme = 'garland'");

  // Make an administrator role
  db_query("INSERT INTO {role} (rid, name) VALUES (3, 'administrator')");
  // Add user 1 to the 'administrator' role
  db_query("INSERT INTO {users_roles} VALUES (1, 3)");
  // Set permissions
  db_query("INSERT INTO permission VALUES (3, 3,'administer blocks, create book pages, create new books, edit book pages, edit own book pages, outline posts in books, see printer-friendly version, access comments, administer comments, post comments, post comments without approval, access erp, admin erp, erp quick find block, admin erp accounting, view erp accounting, access erp cart, admin erp cart, add cash_sale, admin cash_sale, edit cash_sale, edit own cash_sale, erp contact edit, erp contact list, erp contact remove, erp contact save, admin customers, autocomplete customers, create customer, customer quick find block, edit all customers, edit customer, edit own customer, list customers, admin erp_email, send email, admin franchise, add goods_receipt, admin goods_receive, edit goods_receipt, edit own goods_receipt, admin google_maps, view erp google map, add invoice, admin invoice, edit invoice, edit own invoice, view statements, admin item, item cart, item edit, item list, item price list, item wholesale, add job, admin job, edit all jobs, edit job, edit own job, job list, open jobs, admin payment, edit payment, list payments, receive payment, admin pricing, view pricing levels, erp print, add purchase_order, admin purchase_order, edit own purchase_order, edit purchase_order, add quote, admin quote, edit own quote, edit quote, admin stock, stock adjust, stock cart, stock edit, stock list, stock wholesale, admin store, create store, edit all stores, edit own store, edit store, store list, admin supplier, create supplier, edit supplier, supplier autocomplete, supplier list, access erp tax, admin erp tax, create timesheet, edit timesheet, list timesheets, administer filters, administer menu, access content, administer content types, administer nodes, revert revisions, view revisions, administer organic groups, create pdf, administer search, search content, use advanced search, access administration pages, administer site configuration, select different theme, administer taxonomy, upload files, view uploaded files, access user profiles, administer access control, administer users, change own username, access all views, administer views, administer workflow, schedule workflow transitions',0)");

  // Make a staff role
  db_query("INSERT INTO {role} (rid, name) VALUES (4, 'staff')");
  // Add user 1 to the 'admin user' role
  db_query("INSERT INTO {users_roles} VALUES (1, 4)");
  // Set permissions
  db_query("INSERT INTO permission VALUES (4, 4,'create book pages, edit book pages, edit own book pages, see printer-friendly version, access comments, post comments, post comments without approval, access erp, erp quick find block, view erp accounting, access erp cart, add cash_sale, edit cash_sale, edit own cash_sale, erp contact edit, erp contact list, erp contact remove, erp contact save, autocomplete customers, create customer, customer quick find block, edit all customers, edit customer, edit own customer, list customers, send email, add goods_receipt, edit goods_receipt, edit own goods_receipt, view erp google map, add invoice, edit invoice, edit own invoice, view statements, item cart, item edit, item list, item price list, item wholesale, add job, edit all jobs, edit job, edit own job, job list, open jobs, edit payment, list payments, receive payment, view pricing levels, erp print, add purchase_order, edit own purchase_order, edit purchase_order, add quote, edit own quote, edit quote, stock adjust, stock cart, stock edit, stock list, stock wholesale, create store, edit all stores, edit own store, edit store, store list, create supplier, edit supplier, supplier autocomplete, supplier list, access erp tax, create timesheet, edit timesheet, list timesheets, access content, create pdf, search content, use advanced search, upload files, access user profiles',0)");

  // Set some drupal defaults
  variable_set('site_frontpage', 'erp/jobs/my');
  variable_set('erp_accounting_currency_format', '%-12.2f');

  // Turn off all "promoting", drupal things first
  $types = array('book', 'story', 'page',
                 // Now erp things
                 'erp_cash_sale', 'erp_customer', 'erp_goods_receive', 'erp_invoice', 'erp_item',
                 'erp_job', 'erp_payment', 'erp_purchase_order', 'erp_quote', 'erp_store',
                 'erp_supplier', 'erp_timesheet');

  foreach ($types as $type) {
    variable_set('node_options_'. $type, array('status'));
  }

  menu_rebuild();
  _block_rehash();
}

